"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Project: ngrx-quiz-api (https://github.com/pmstss/ngrx-quiz-api)
 * Copyright 2019 Viachaslau Tyshkavets
 * Licensed under the GPLv3 License. See LICENSE.txt in the project root for license information.
 */
const mongoose = require("mongoose");
const quiz_item_model_1 = require("../quiz-item/quiz-item-model");
const api_error_1 = require("../../api/api-error");
class AdminQuizItemRepo {
    getItem(itemId) {
        return quiz_item_model_1.QuizItemModel
            .aggregate()
            .match({
            _id: mongoose.Types.ObjectId(itemId)
        })
            .addFields({
            id: '$_id',
            choices: {
                $map: {
                    input: '$choices',
                    as: 'x',
                    in: {
                        $mergeObjects: ['$$x', { id: '$$x._id' }]
                    }
                }
            }
        })
            .project({
            _id: 0,
            __v: 0,
            'choices._id': 0
        })
            .exec()
            .then((res) => {
            if (res.length) {
                return res[0];
            }
            throw new api_error_1.ApiError('No such item', 404);
        });
    }
    createItem(quizId, item) {
        return quiz_item_model_1.QuizItemModel.create({
            quizId: mongoose.Types.ObjectId(quizId),
            question: item.question,
            choices: item.choices,
            singleChoice: item.singleChoice,
            randomizeChoices: item.randomizeChoices,
            counter: 0,
            order: 100
        }).then((doc) => this.getItem(doc._id));
    }
    updateItem(itemId, item) {
        return quiz_item_model_1.QuizItemModel.findOneAndUpdate({
            _id: mongoose.Types.ObjectId(itemId)
        }, {
            $set: {
                question: item.question,
                choices: item.choices
                    .map((ch) => (Object.assign({}, ch, { _id: mongoose.Types.ObjectId(ch.id) }))),
                singleChoice: item.singleChoice,
                randomizeChoices: item.randomizeChoices
            }
        }, {
            new: true
        }).exec().then((doc) => this.getItem(doc._id));
    }
    deleteItem(itemId) {
        return quiz_item_model_1.QuizItemModel.findOneAndRemove({
            _id: mongoose.Types.ObjectId(itemId)
        }).exec().then((res) => {
            if (!res) {
                throw new api_error_1.ApiError('No such item', 404);
            }
        });
    }
    updateQuizItemsOrder(quizId, itemIds) {
        // TODO ### is it possible to perform in single query?
        return Promise.all(itemIds.map((itemId, idx) => {
            return quiz_item_model_1.QuizItemModel.update({
                _id: mongoose.Types.ObjectId(itemId),
                quizId: mongoose.Types.ObjectId(quizId)
            }, {
                $set: {
                    order: idx
                }
            }).exec();
        })).then((res) => {
            if (!res.every((r) => !!r.ok)) {
                throw new api_error_1.ApiError('Order update error', 501);
            }
        });
    }
}
exports.AdminQuizItemRepo = AdminQuizItemRepo;
