"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Project: ngrx-quiz-api (https://github.com/pmstss/ngrx-quiz-api)
 * Copyright 2019 Viachaslau Tyshkavets
 * Licensed under the GPLv3 License. See LICENSE.txt in the project root for license information.
 */
const mongoose = require("mongoose");
const api_error_1 = require("../../api/api-error");
const quiz_model_1 = require("../quiz/quiz-model");
const quiz_item_model_1 = require("../quiz-item/quiz-item-model");
class AdminQuizRepo {
    getQuizList(user) {
        const criteria = this.getQuizMatchCriteria(null, user);
        delete criteria._id;
        return quiz_model_1.QuizModel
            .aggregate()
            .match(criteria)
            .lookup({
            from: 'items',
            localField: '_id',
            foreignField: 'quizId',
            as: 'items'
        })
            .addFields({
            id: '$_id',
            totalQuestions: { $size: '$items' }
        })
            .project({
            _id: 0,
            __v: 0,
            items: 0
        })
            .exec();
    }
    getQuizMatchCriteria(quizId, user) {
        const criteria = {
            _id: mongoose.Types.ObjectId(quizId)
        };
        if (!user.admin) {
            criteria.userId = user.id;
        }
        return criteria;
    }
    getQuiz(quizId, user) {
        return quiz_model_1.QuizModel
            .aggregate()
            .match(this.getQuizMatchCriteria(quizId, user))
            .lookup({
            from: 'items',
            let: { id: '$_id' },
            pipeline: [{
                    $match: {
                        $expr: {
                            $eq: ['$quizId', '$$id']
                        }
                    }
                }],
            as: 'items'
        })
            // adding "id" instead of "_id" for items and their inner choices
            .addFields({
            items: {
                $map: {
                    input: '$items',
                    as: 'x',
                    in: {
                        $mergeObjects: [
                            '$$x',
                            {
                                id: '$$x._id',
                                choices: {
                                    $map: {
                                        input: '$$x.choices',
                                        as: 'y',
                                        in: {
                                            $mergeObjects: [
                                                '$$y',
                                                { id: '$$y._id' }
                                            ]
                                        }
                                    }
                                }
                            }
                        ]
                    }
                }
            },
            id: '$_id'
        })
            // sorting by items order
            .unwind({ path: '$items', preserveNullAndEmptyArrays: true })
            .sort({ 'items.order': 1, 'items.updatedAt': 1 })
            .group({
            _id: '$_id',
            items: {
                $push: '$items'
            },
            quizMeta: {
                $mergeObjects: '$$CURRENT'
            }
        })
            .project({
            'quizMeta.items': 0
        })
            .replaceRoot({
            $mergeObjects: ['$quizMeta', { items: '$items' }]
        })
            // removing aux fields
            .project({
            _id: 0,
            __v: 0,
            'items._id': 0,
            'items.__v': 0,
            'items.choices._id': 0
        })
            .exec()
            .then((res) => {
            if (res.length) {
                return res[0];
            }
            throw new api_error_1.ApiError('No such quiz', 404);
        });
    }
    createQuiz(quiz, user) {
        return quiz_model_1.QuizModel.create({
            shortName: quiz.shortName,
            name: quiz.name,
            description: quiz.name,
            descriptionFull: quiz.name,
            randomizeItems: quiz.randomizeItems,
            public: true,
            published: false,
            userId: user.id
        }).then((doc) => this.getQuiz(doc._id, user));
    }
    deleteQuiz(quizId, user) {
        return quiz_model_1.QuizModel.deleteOne(this.getQuizMatchCriteria(quizId, user)).exec()
            .then((quizDeleteResult) => {
            if (!quizDeleteResult.ok || quizDeleteResult.n === 0) {
                throw new api_error_1.ApiError('Error deleting quiz', 404);
            }
        })
            .then(() => {
            return quiz_item_model_1.QuizItemModel.deleteMany({ quizId: mongoose.Types.ObjectId(quizId) }).exec();
        }).then((itemsDeleteResult) => {
            if (!itemsDeleteResult.ok) {
                throw new api_error_1.ApiError('Error deleting quiz', 404);
            }
        });
    }
    updateQuiz(quizId, quiz, user) {
        return quiz_model_1.QuizModel.findOneAndUpdate(this.getQuizMatchCriteria(quizId, user), {
            $set: {
                shortName: quiz.shortName,
                name: quiz.name,
                description: quiz.description,
                descriptionFull: quiz.descriptionFull,
                randomizeItems: quiz.randomizeItems
            }
        }).exec()
            .then(() => this.getQuiz(quizId, user));
    }
    publishQuiz(quizId, user) {
        return quiz_model_1.QuizModel.findOneAndUpdate(this.getQuizMatchCriteria(quizId, user), {
            $set: {
                published: true
            }
        }).exec()
            .then((doc) => !!doc);
    }
    unpublishQuiz(quizId, user) {
        return quiz_model_1.QuizModel.findOneAndUpdate(this.getQuizMatchCriteria(quizId, user), {
            $set: {
                published: false
            }
        }).exec()
            .then((doc) => !!doc);
    }
}
exports.AdminQuizRepo = AdminQuizRepo;
