"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bcrypt_1 = require("bcrypt");
const response_writer_1 = require("../../api/response-writer");
const api_error_1 = require("../../api/api-error");
const token_utils_1 = require("../../token/token-utils");
const recaptcha_1 = require("../../utils/recaptcha");
const random_1 = require("../../utils/random");
const mailer_1 = require("../../mail/mailer");
const mail_result_repo_1 = require("../mail/mail-result-repo");
const ban_words_1 = require("../../utils/ban-words");
class AuthController {
    constructor(repo, tokenRepo) {
        this.repo = repo;
        this.tokenRepo = tokenRepo;
    }
    login(req, res, next) {
        if (!req.body.captchaToken || !req.body.email || !req.body.password) {
            return response_writer_1.writeErrorResponse(res, next, new api_error_1.ApiError('Invalid parameters', 422));
        }
        return response_writer_1.writeResponse(recaptcha_1.verifyRecaptcha(req.body.captchaToken).then((valid) => {
            if (!valid) {
                throw new api_error_1.ApiError('Invalid captcha', 403);
            }
            return this.loginAux(req.body.email, req.body.password);
        }), req, res, next);
    }
    loginAux(email, password) {
        return this.repo.getUser(email).then((user) => {
            if (user && bcrypt_1.compareSync(password, user.password)) {
                const token = token_utils_1.TokenUtils.createUserToken(user);
                return this.tokenRepo.storeUserToken(user.id, token)
                    .then(() => ({ token }));
            }
            throw new api_error_1.ApiError('Invalid credentials', 401);
        });
    }
    logout(req, res, next) {
        return response_writer_1.writeResponse(token_utils_1.TokenUtils.verifyIgnoringExpiration(token_utils_1.TokenUtils.parseFromRequest(req))
            .then(tokenData => this.tokenRepo.removeUserToken(tokenData.user.id))
            .then(() => null)
            .catch(() => {
            console.warn('Invalid token for user'); // TODO log somewhere?
        }), req, res, next);
    }
    register(req, res, next) {
        if (!req.body.captchaToken || !req.body.email || !req.body.password || !req.body.fullName) {
            return response_writer_1.writeErrorResponse(res, next, new api_error_1.ApiError('Invalid parameters', 422));
        }
        if (ban_words_1.BanWords.isInsulting(req.body.fullName)) {
            return response_writer_1.writeErrorResponse(res, next, new api_error_1.ApiError('Please be polite or banned', 422));
        }
        return response_writer_1.writeResponse(recaptcha_1.verifyRecaptcha(req.body.captchaToken).then((valid) => {
            if (!valid) {
                throw new api_error_1.ApiError('Invalid captcha', 403);
            }
            return this.repo.createUser({
                id: null,
                email: req.body.email,
                fullName: req.body.fullName,
                password: req.body.password,
                admin: false,
                social: null,
                anonymous: false
            }).then(() => this.loginAux(req.body.email, req.body.password));
        }), req, res, next);
    }
    refreshToken(req, res, next) {
        if (!req.body.token) {
            return response_writer_1.writeErrorResponse(res, next, new api_error_1.ApiError('Missing token', 403));
        }
        const token = req.body.token;
        return response_writer_1.writeResponse(token_utils_1.TokenUtils.verifyIgnoringExpiration(token)
            .then(tokenData => this.tokenRepo.exists(tokenData.user.id, token)
            .then(exists => ([tokenData, exists])))
            .then(([tokenData, exists]) => {
            if (exists) {
                const token = token_utils_1.TokenUtils.createUserToken(tokenData.user);
                return this.tokenRepo.storeUserToken(tokenData.user.id, token)
                    .then(() => ({ token }));
            }
            throw new api_error_1.ApiError('Invalid token', 403);
        }), req, res, next);
    }
    anonymousLogin(req, res, next) {
        if (!req.body.captchaToken) {
            return response_writer_1.writeErrorResponse(res, next, new api_error_1.ApiError('Invalid parameters', 422));
        }
        return response_writer_1.writeResponse(recaptcha_1.verifyRecaptcha(req.body.captchaToken).then((valid) => {
            if (!valid) {
                throw new api_error_1.ApiError('Invalid captcha', 403);
            }
            const email = `anonym-${random_1.RandomUtils.generateAlfaNum(12, 32)}@rankme.pro`;
            return this.repo.createUser({
                email,
                id: null,
                fullName: 'Anonymous',
                password: random_1.RandomUtils.generateAlfaNum(12, 32),
                admin: false,
                social: null,
                anonymous: true
            })
                .then(() => {
                return this.repo.getUser(email);
            })
                .then((user) => {
                const token = token_utils_1.TokenUtils.createUserToken(user);
                return this.tokenRepo.storeUserToken(user.id, token)
                    .then(() => ({ token }));
            });
        }), req, res, next);
    }
    requestResetPassToken(req, res, next) {
        if (!req.body.captchaToken || !req.body.email) {
            return response_writer_1.writeErrorResponse(res, next, new api_error_1.ApiError('Invalid parameters', 422));
        }
        return response_writer_1.writeResponse(recaptcha_1.verifyRecaptcha(req.body.captchaToken).then((valid) => {
            if (!valid) {
                throw new api_error_1.ApiError('Invalid captcha', 403);
            }
            const token = random_1.RandomUtils.generateAlfaNum(32, 32);
            return this.repo.upsertPasswordToken(req.body.email, token).then((user) => {
                if (!user) {
                    throw new api_error_1.ApiError('No such user', 403); // TODO do not expose this info?
                }
                return mailer_1.Mailer.getInstance().sendPasswordReset(user.fullName, req.body.email, token)
                    .then((mailResult) => {
                    return mail_result_repo_1.MailResultRepo.insertMailResult(user.id, mailResult);
                });
            });
        }), req, res, next);
    }
    resetPass(req, res, next) {
        if (!req.body.token || !req.body.password) {
            return response_writer_1.writeErrorResponse(res, next, new api_error_1.ApiError('Invalid parameters', 422));
        }
        return response_writer_1.writeResponse(this.repo.updatePassword(req.body.token, req.body.password)
            .then((user) => {
            if (!user) {
                throw new api_error_1.ApiError('No such user', 403); // TODO do not expose this info?
            }
            return this.loginAux(user.email, req.body.password);
        }), req, res, next);
    }
}
exports.AuthController = AuthController;
