"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fetch = require('node-fetch');
const token_utils_1 = require("../../token/token-utils");
const response_writer_1 = require("../../api/response-writer");
const api_error_1 = require("../../api/api-error");
const consts_1 = require("../../consts/consts");
const oauth_token_model_1 = require("../token/oauth-token-model");
class OAuthController {
    constructor(tokenRepo, authRepo) {
        this.tokenRepo = tokenRepo;
        this.authRepo = authRepo;
    }
    getBaseUrl(req) {
        return `${req.protocol}://${req.get('host')}`;
    }
    google(req, res, next) {
        const code = req.query.code;
        if (!code) {
            return response_writer_1.writeErrorResponse(res, next, new api_error_1.ApiError('No code in Google response', 422));
        }
        const BASE_URL = this.getBaseUrl(req);
        const params = new URLSearchParams();
        params.append('code', code);
        params.append('clientId', consts_1.OAUTH_GOOGLE_CLIENT_ID);
        params.append('clientSecret', consts_1.OAUTH_GOOGLE_CLIENT_SECRET);
        params.append('redirect_uri', `${BASE_URL}/oauth/google`);
        params.append('grant_type', 'authorization_code');
        fetch('https://www.googleapis.com/oauth2/v4/token', {
            method: 'POST',
            body: params
        })
            .then((response) => response.json())
            .then((tokenResponse) => fetch(`https://oauth2.googleapis.com/tokeninfo?id_token=${tokenResponse.id_token}`))
            .then((response) => response.json())
            .then((tokenInfo) => this.tokenRepo.storeOauthToken(tokenInfo, oauth_token_model_1.OAuthType.GOOGLE))
            .then((user) => this.createUser(user))
            .then((user) => {
            const token = token_utils_1.TokenUtils.createUserToken(user);
            return this.tokenRepo.storeUserToken(user.id, token).then(() => token);
        })
            .then((token) => res.redirect(`${consts_1.OAUTH_TOKEN_CALLBACK_URL}${token}`))
            .catch((err) => {
            console.error(err);
            response_writer_1.writeErrorResponse(res, next, new api_error_1.ApiError('OAuth Error', 400));
        });
    }
    github(req, res, next) {
        const code = req.query.code;
        if (!code) {
            return response_writer_1.writeErrorResponse(res, next, new api_error_1.ApiError('No code in GitHub response', 422));
        }
        const BASE_URL = this.getBaseUrl(req);
        const params = new URLSearchParams();
        params.append('code', code);
        params.append('client_id', consts_1.OAUTH_GITHUB_CLIENT_ID);
        params.append('client_secret', consts_1.OAUTH_GITHUB_CLIENT_SECRET);
        params.append('redirect_uri', `${BASE_URL}/oauth/github`);
        fetch('https://github.com/login/oauth/access_token', {
            method: 'POST',
            body: params
        })
            .then((response) => response.text())
            .then((text) => text.split('&').reduce((res, item) => {
            const [key, value] = item.split('=');
            res[key] = value;
            return res;
        }, {})).then((tokenResponse) => fetch(`https://api.github.com/user?access_token=${tokenResponse.access_token}`))
            .then((response) => response.json())
            .then((tokenInfo) => this.tokenRepo.storeOauthToken(tokenInfo, oauth_token_model_1.OAuthType.GITHUB))
            .then((user) => this.createUser(user))
            .then((user) => {
            const token = token_utils_1.TokenUtils.createUserToken(user);
            return this.tokenRepo.storeUserToken(user.id, token).then(() => token);
        })
            .then((token) => res.redirect(`${consts_1.OAUTH_TOKEN_CALLBACK_URL}${token}`))
            .catch((err) => {
            console.error(err);
            response_writer_1.writeErrorResponse(res, next, new api_error_1.ApiError('OAuth Error', 400));
        });
    }
    createUser(user) {
        return this.authRepo.getUser(user.email, user.social).then((u) => {
            if (u) {
                return u;
            }
            return this.authRepo.createUser(user);
        });
    }
}
exports.OAuthController = OAuthController;
