"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Project: ngrx-quiz-api (https://github.com/pmstss/ngrx-quiz-api)
 * Copyright 2019 Viachaslau Tyshkavets
 * Licensed under the GPLv3 License. See LICENSE.txt in the project root for license information.
 */
const mongoose = require("mongoose");
class StateService {
    constructor(req) {
        this.req = req;
        this.state = req.session;
        if (!this.state.quizes) {
            this.state.quizes = {};
        }
    }
    getQuizState(quizId) {
        return this.state.quizes[quizId];
    }
    getIdByName(quizName) {
        return Object.keys(this.state.quizes).find(id => this.state.quizes[id].shortName === quizName);
    }
    hasQuizState(quizId) {
        return this.state.quizes.hasOwnProperty(quizId);
    }
    initQuizState(quiz) {
        if (!this.hasQuizState(quiz.id)) {
            this.state.quizes[quiz.id] = {
                itemIds: quiz.itemIds,
                shortName: quiz.shortName,
                answers: {},
                score: 0,
                scoreSaved: false,
                startDate: new Date()
            };
        }
    }
    resetQuizState(quizId) {
        if (this.hasQuizState(quizId)) {
            this.state.quizes[quizId] = Object.assign({}, this.state.quizes[quizId], { answers: {}, score: 0, scoreSaved: false, startDate: new Date() });
        }
    }
    getClientQuizState(quizId) {
        const quizState = this.getQuizState(quizId);
        return {
            answers: quizState.answers
        };
    }
    getAnsweredCount(quizId) {
        const quizState = this.getQuizState(quizId);
        return quizState ? Object.keys(quizState.answers).length : 0;
    }
    getTotalQuestions(quizId) {
        const quizState = this.getQuizState(quizId);
        return quizState ? quizState.itemIds.length : 0;
    }
    isStarted(quizId) {
        return this.getAnsweredCount(quizId) > 0;
    }
    isFinished(quizId) {
        const total = this.getTotalQuestions(quizId);
        return total > 0 && this.getAnsweredCount(quizId) === total;
    }
    isAnswered(quizId, itemId) {
        const quizState = this.getQuizState(quizId);
        return quizState.answers.hasOwnProperty(itemId);
    }
    getItemAnswer(quizId, itemId) {
        return this.getQuizState(quizId).answers[itemId];
    }
    addAnswer(quizId, itemId, answerResult) {
        const quizState = this.getQuizState(quizId);
        if (!this.isStarted(quizId)) {
            quizState.startDate = new Date();
        }
        quizState.answers[itemId] = answerResult;
        if (answerResult.correct) {
            quizState.score++;
        }
    }
    isScoreSaved(quizId) {
        const quizState = this.getQuizState(quizId);
        return !quizState || quizState.scoreSaved;
    }
    isScoreSaveRequired(quizId) {
        return this.isFinished(quizId) && !this.isScoreSaved(quizId);
    }
    getQuizScoreDoc(quizId) {
        const quizState = this.getQuizState(quizId);
        const userId = this.req.tokenData && this.req.tokenData.user && this.req.tokenData.user.id;
        return {
            quizId: new mongoose.Types.ObjectId(quizId),
            userId: userId ? new mongoose.Types.ObjectId(userId) : null,
            sessionId: this.req.sessionID,
            score: quizState.score / quizState.itemIds.length
        };
    }
}
exports.StateService = StateService;
